expanded class
	MPZ

inherit
	P_AORS_STATIC      undefine default_create, copy, is_equal, out end
	P_AORS_UI_STATIC   undefine default_create, copy, is_equal, out end
	P_CMP_STATIC       undefine default_create, copy, is_equal, out end
	P_GET_STR_STATIC   undefine default_create, copy, is_equal, out end
	P_GMP_STATIC       undefine default_create, copy, is_equal, out end
	P_MUL_STATIC       undefine default_create, copy, is_equal, out end
	P_MUL_I_STATIC     undefine default_create, copy, is_equal, out end
	P_NEG_STATIC       undefine default_create, copy, is_equal, out end
	P_POW_UI_STATIC    undefine default_create, copy, is_equal, out end
	P_SET_SI_STATIC    undefine default_create, copy, is_equal, out end
	P_SET_STATIC       undefine default_create, copy, is_equal, out end
	P_SET_STR_STATIC   undefine default_create, copy, is_equal, out end
	P_SET_UI_STATIC    undefine default_create, copy, is_equal, out end
	P_TDIV_Q_STATIC    undefine default_create, copy, is_equal, out end
	P_TDIV_Q_UI_STATIC undefine default_create, copy, is_equal, out end
	COMPARABLE         undefine default_create, copy, is_equal, out end

feature {NONE}

	default_create
		do
			create mpz
		end

feature {MPZ}

	mpz: MPZ_REF

feature {ANY}

	set_from_string (a_init: STRING_8)
		local
			l_s: C_STRING
		do
			create l_s.make (a_init)
			gmpz_set_str (mpz, create {CE_INTEGER_8_POINTER}.make_shared (l_s.item), 10).do_nothing
		end

	set_from_integer_32 (a_init: INTEGER_32)
		do
			gmpz_set_si (mpz, a_init)
		end

	set_from_natural_32 (a_init: NATURAL_32)
		do
			gmpz_set_ui (mpz, a_init)
		end

feature {ANY}

	plus alias "+" (a_other: MPZ): MPZ
		do
			gmpz_add (Result.mpz, mpz, a_other.mpz)
		end

	minus alias "-" (a_other: MPZ): MPZ
		do
			gmpz_sub (Result.mpz, mpz, a_other.mpz)
		end

	product alias "*" (a_other: MPZ): MPZ
		do
			gmpz_mul (Result.mpz, mpz, a_other.mpz)
		end

	quotient alias "/" (a_other: MPZ): MPZ
		do
			gmpz_tdiv_q (Result.mpz, mpz, a_other.mpz)
		end

	power alias "^" (a_other: NATURAL_32): MPZ
		do
			gmpz_pow_ui (Result.mpz, mpz, a_other)
		end

	opposite alias "-": MPZ
		do
			 gmpz_neg (Result.mpz, mpz)
		end

	sign: INTEGER_32
		do
			Result := mpz.item.mp_size.sign
		end

feature {ANY}

	plus_natural_32 alias "|+|" (a_other: NATURAL_32): MPZ
		do
			gmpz_add_ui (Result.mpz, mpz, a_other)
		end

	minus_natural_32 alias "|-|" (a_other: NATURAL_32): MPZ
		do
			gmpz_sub_ui (Result.mpz, mpz, a_other)
		end

	product_natural_32 alias "|*|" (a_other: NATURAL_32): MPZ
		do
			gmpz_mul_ui (Result.mpz, mpz, a_other)
		end

	quotient_natural_32 alias "|/|" (a_other: NATURAL_32): MPZ
		do
			gmpz_tdiv_q_ui (Result.mpz, mpz, a_other).do_nothing
		end

feature {ANY}

	fits_into_natural_32: BOOLEAN
		do
			Result := gmpz_fits_uint_p (mpz) /= 0
		end

	to_natural_32: NATURAL_32
		require
			fits_into_natural_32
		do
			Result := gmpz_get_ui (mpz)
		end

feature {ANY}

	copy (a_other: MPZ)
		do
			default_create
			gmpz_set (mpz, a_other.mpz)
		end

	is_less alias "<" (a_other: MPZ): BOOLEAN
		do
			Result := gmpz_cmp (mpz, a_other.mpz) < 0
		end

	is_equal (a_other: MPZ): BOOLEAN
		do
			Result := gmpz_cmp (mpz, a_other.mpz) = 0
		end

	out: STRING_8
		local
			l_s: CE_POINTER [INTEGER_8]
		do
			l_s := gmpz_get_str (create {CE_INTEGER_8_POINTER}.make, 10, mpz)
			Result := eif_string (l_s)
			l_s.c_pointer.memory_free
		end

end
